﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.GridView;

namespace TreeGridHybrid
{
	public partial class ExpandableCellNode : UserControl
	{
		public ExpandableCellNode()
		{
			InitializeComponent();
			this.MouseLeftButtonDown += new MouseButtonEventHandler(ExpandableCellNode_MouseLeftButtonDown);
			this.Loaded += new RoutedEventHandler(ExpandableCellNode_Loaded);
		}

		public GridViewCell ParentCell 
		{
			get
			{
				return ((GridViewCell) VisualTreeHelper.GetParent(this));
			}
		}

		public Person Person
		{
			get
			{
				return this.ParentCell.DataContext as Person;
			}
		}

		void ExpandableCellNode_Loaded(object sender, RoutedEventArgs e)
		{
			SetVisibility();
			string plusMinusSign = this.Person.IsExpanded ? "-" : "+";
			this.ExpandElement.Margin = new Thickness(GetIndent(this.Person), 0, 0, 0);
			this.ContentTextBlock.Text = this.Person.FirstName + " " + this.Person.LastName;
		}

		public void SetVisibility()
		{
			if (HasCollapsedAncestor(this.Person))
			{
				this.ParentCell.ParentRow.Height = 0;
				this.ParentCell.ParentRow.MinHeight = 0;
			}
		}

		private bool HasCollapsedAncestor(Person person)
		{
			bool isParentCollapsed = false;

			Person p = person;
			while (p.Parent != null)
			{
				if (!p.Parent.IsExpanded)
					isParentCollapsed = true;
				p = p.Parent;
			}

			return isParentCollapsed;
		}

		private double GetIndent(Person person)
		{
			int count = 0;
			Person p = person;
			while (p.Parent != null)
			{
				count++;
				p = p.Parent;
			}

			return count * 15;
		}

		void ExpandableCellNode_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			this.Person.IsExpanded = !this.Person.IsExpanded;
			
			string plusMinusSign = this.Person.IsExpanded?"-":"+";
			this.PlusMinusTextBlock.Text = plusMinusSign;

			UpdateRowsVisibility();

		}

		private void UpdateRowsVisibility()
		{
			foreach (GridViewRow row in this.ParentCell.ParentRow.ParentGrid.ItemsControl.VirtualizingPanel.Children)
			{
				Person p = row.DataContext as Person;
				if (p.Parent != null && HasCollapsedAncestor(p))
				{
					row.Height = 0;
					row.MinHeight = 0;
				}
				else
				{
					row.MinHeight = 20;
				}
			}
		}
	}
}
